==> elinks_0.13~20190125.orig.tar.xz: elinks-0.13~20190125/src/intl/gettext/finddomain.c <==
/* Handle list of needed message catalogs
   Copyright (C) 1995-1999, 2000, 2001 Free Software Foundation, Inc.
   Written by Ulrich Drepper <drepper@gnu.org>, 1995.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>

#if defined HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "elinks.h"

#include "intl/gettext/gettextP.h"
#include "intl/gettext/libintl.h"
#include "util/string.h"

/* List of already loaded domains.  */
static struct loaded_l10nfile *_nl_loaded_domains;

/* Return a data structure describing the message catalog described by
   the DOMAINNAME and CATEGORY parameters with respect to the currently
   established bindings.  */
struct loaded_l10nfile *
_nl_find_domain(const unsigned char *dirname, unsigned char *locale, const unsigned char *domainname,
		struct binding *domainbinding)
{
	struct loaded_l10nfile *retval;
	const unsigned char *language;
	const unsigned char *modifier;
	const unsigned char *territory;
	const unsigned char *codeset;
	const unsigned char *normalized_codeset;
	const unsigned char *special;
	const unsigned char *sponsor;
	const unsigned char *revision;
	const unsigned char *alias_value;
	int mask;

	/* LOCALE can consist of up to four recognized parts for the XPG syntax:

	   language[_territory[.codeset]][@modifier]

	   and six parts for the CEN syntax:

	   language[_territory][+audience][+special][,[sponsor][_revision]]

	   Beside the first part all of them are allowed to be missing.  If
	   the full specified locale is not found, the less specific one are
	   looked for.  The various parts will be stripped off according to
	   the following order:
	   (1) revision
	   (2) sponsor
	   (3) special
	   (4) codeset
	   (5) normalized codeset
	   (6) territory
	   (7) audience/modifier
	 */

	/* If we have already tested for this locale entry there has to
	   be one data set in the list of loaded domains.  */
	retval = _nl_make_l10nflist(&_nl_loaded_domains, dirname,
				    strlen(dirname) + 1, 0, locale, NULL, NULL,
				    NULL, NULL, NULL, NULL, NULL, domainname,
				    0);
	if (retval != NULL) {
		/* We know something about this locale.  */
		int cnt;

		if (retval->decided == 0)
			_nl_load_domain(retval, domainbinding);

		if (retval->data != NULL)
			return retval;

		for (cnt = 0; retval->successor[cnt] != NULL; ++cnt) {
			if (retval->successor[cnt]->decided == 0)
				_nl_load_domain(retval->successor[cnt],
						domainbinding);

			if (retval->successor[cnt]->data != NULL)
				break;
		}
		return cnt >= 0 ? retval : NULL;
		/* NOTREACHED */
	}

	/* See whether the locale value is an alias.  If yes its value
	 *overwrites* the alias name.  No test for the original value is
	 done.  */
	alias_value = _nl_expand_alias(locale);
	if (alias_value != NULL) {
		locale = strdup(alias_value);
		if (locale == NULL)
			return NULL;
	}

	/* Now we determine the single parts of the locale name.  First
	   look for the language.  Termination symbols are `_' and `@' if
	   we use XPG4 style, and `_', `+', and `,' if we use CEN syntax.  */
	mask = _nl_explode_name(locale, &language, &modifier, &territory,
				&codeset, &normalized_codeset, &special,
				&sponsor, &revision);

	/* Create all possible locale entries which might be interested in
	   generalization.  */
	retval = _nl_make_l10nflist(&_nl_loaded_domains, dirname,
				    strlen(dirname) + 1, mask, language,
				    territory, codeset, normalized_codeset,
				    modifier, special, sponsor, revision,
				    domainname, 1);
	if (retval == NULL)
		/* This means we are out of core.  */
		return NULL;

	if (retval->decided == 0)
		_nl_load_domain(retval, domainbinding);
	if (retval->data == NULL) {
		int cnt;

		for (cnt = 0; retval->successor[cnt] != NULL; ++cnt) {
			if (retval->successor[cnt]->decided == 0)
				_nl_load_domain(retval->successor[cnt],
						domainbinding);
			if (retval->successor[cnt]->data != NULL)
				break;
		}
	}

	/* The room for an alias was dynamically allocated.  Free it now.  */
	if (alias_value != NULL)
		free(locale);

	/* The space for normalized_codeset is dynamically allocated.  Free it.  */
	if (mask & XPG_NORM_CODESET)
		free((void *) normalized_codeset);

	return retval;
}

==> elinks_0.16.1.1.orig.tar.xz: elinks-0.16.1.1/src/intl/gettext/finddomain.c <==
/* Handle list of needed message catalogs
   Copyright (C) 1995-1999, 2000, 2001 Free Software Foundation, Inc.
   Written by Ulrich Drepper <drepper@gnu.org>, 1995.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>

#if defined HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "elinks.h"

#include "intl/gettext/gettextP.h"
#include "intl/libintl.h"
#include "util/string.h"

/* List of already loaded domains.  */
static struct loaded_l10nfile *_nl_loaded_domains;

/* Return a data structure describing the message catalog described by
   the DOMAINNAME and CATEGORY parameters with respect to the currently
   established bindings.  */
struct loaded_l10nfile *
_nl_find_domain(const char *dirname, char *locale, const char *domainname,
		struct binding *domainbinding)
{
	struct loaded_l10nfile *retval;
	const char *language;
	const char *modifier;
	const char *territory;
	const char *codeset;
	const char *normalized_codeset;
	const char *special;
	const char *sponsor;
	const char *revision;
	const char *alias_value;
	int mask;

	/* LOCALE can consist of up to four recognized parts for the XPG syntax:

	   language[_territory[.codeset]][@modifier]

	   and six parts for the CEN syntax:

	   language[_territory][+audience][+special][,[sponsor][_revision]]

	   Beside the first part all of them are allowed to be missing.  If
	   the full specified locale is not found, the less specific one are
	   looked for.  The various parts will be stripped off according to
	   the following order:
	   (1) revision
	   (2) sponsor
	   (3) special
	   (4) codeset
	   (5) normalized codeset
	   (6) territory
	   (7) audience/modifier
	 */

	/* If we have already tested for this locale entry there has to
	   be one data set in the list of loaded domains.  */
	retval = _nl_make_l10nflist(&_nl_loaded_domains, dirname,
				    strlen(dirname) + 1, 0, locale, NULL, NULL,
				    NULL, NULL, NULL, NULL, NULL, domainname,
				    0);
	if (retval != NULL) {
		/* We know something about this locale.  */
		int cnt;

		if (retval->decided == 0)
			_nl_load_domain(retval, domainbinding);

		if (retval->data != NULL)
			return retval;

		for (cnt = 0; retval->successor[cnt] != NULL; ++cnt) {
			if (retval->successor[cnt]->decided == 0)
				_nl_load_domain(retval->successor[cnt],
						domainbinding);

			if (retval->successor[cnt]->data != NULL)
				break;
		}
		return cnt >= 0 ? retval : NULL;
		/* NOTREACHED */
	}

	/* See whether the locale value is an alias.  If yes its value
	 *overwrites* the alias name.  No test for the original value is
	 done.  */
	alias_value = _nl_expand_alias(locale);
	if (alias_value != NULL) {
		locale = strdup(alias_value);
		if (locale == NULL)
			return NULL;
	}

	/* Now we determine the single parts of the locale name.  First
	   look for the language.  Termination symbols are `_' and `@' if
	   we use XPG4 style, and `_', `+', and `,' if we use CEN syntax.  */
	mask = _nl_explode_name(locale, &language, &modifier, &territory,
				&codeset, &normalized_codeset, &special,
				&sponsor, &revision);

	/* Create all possible locale entries which might be interested in
	   generalization.  */
	retval = _nl_make_l10nflist(&_nl_loaded_domains, dirname,
				    strlen(dirname) + 1, mask, language,
				    territory, codeset, normalized_codeset,
				    modifier, special, sponsor, revision,
				    domainname, 1);
	if (retval == NULL)
		/* This means we are out of core.  */
		return NULL;

	if (retval->decided == 0)
		_nl_load_domain(retval, domainbinding);
	if (retval->data == NULL) {
		int cnt;

		for (cnt = 0; retval->successor[cnt] != NULL; ++cnt) {
			if (retval->successor[cnt]->decided == 0)
				_nl_load_domain(retval->successor[cnt],
						domainbinding);
			if (retval->successor[cnt]->data != NULL)
				break;
		}
	}

	/* The room for an alias was dynamically allocated.  Free it now.  */
	if (alias_value != NULL)
		free(locale);

	/* The space for normalized_codeset is dynamically allocated.  Free it.  */
	if (mask & XPG_NORM_CODESET)
		free((void *) normalized_codeset);

	return retval;
}
